# -*- coding: utf-8 -*-
#===============================================================================
# V0.29
# This is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2, or (at your option) any later
# version.
#===============================================================================

from __init__ import _
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox

from Components.ActionMap import ActionMap
from Components.Button import Button
from Components.MenuList import MenuList
from Components.ConfigList import ConfigListScreen
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText

from Components.config import config
from Components.config import NoSave
from Components.config import ConfigIP
from Components.config import ConfigText
from Components.config import ConfigYesNo
from Components.config import ConfigInteger
from Components.config import ConfigPassword
from Components.config import ConfigSubsection
from Components.config import getConfigListEntry

from enigma import eListboxPythonMultiContent, eListbox, getDesktop, gFont, RT_HALIGN_LEFT, RT_VALIGN_CENTER, ePoint
from Tools.LoadPixmap import LoadPixmap
from Tools.Directories import resolveFilename, SCOPE_CURRENT_PLUGIN, SCOPE_SKIN

config.plugins.NcamStatus  = ConfigSubsection()
config.plugins.NcamStatus.lastServer = ConfigInteger(default = 0)
config.plugins.NcamStatus.extMenu = ConfigYesNo(default = True)
config.plugins.NcamStatus.xOffset = ConfigInteger(default = 50, limits=(0,100))
config.plugins.NcamStatus.useECM = ConfigYesNo(default = False)
config.plugins.NcamStatus.useIP = ConfigYesNo(default = True)
config.plugins.NcamStatus.usePicons = ConfigYesNo(default = False)
config.plugins.NcamStatus.PiconPath = ConfigText(default = resolveFilename(SCOPE_SKIN,"picon_50x30"), fixed_size = False, visible_width=40)

# export Variablen...
LASTSERVER = config.plugins.NcamStatus.lastServer
EXTMENU = config.plugins.NcamStatus.extMenu
XOFFSET = config.plugins.NcamStatus.xOffset
USEECM = config.plugins.NcamStatus.useECM
USEPICONS = config.plugins.NcamStatus.usePicons
PICONPATH = config.plugins.NcamStatus.PiconPath
picons = None


def dlg_xh(w):
	x = getDesktop(0).size().width() - w - XOFFSET.value
	if x < 0: x = 0
	h = getDesktop(0).size().height()
	return x, h

class globalsConfigScreen(Screen, ConfigListScreen):
	skin = """
		<screen flags="wfNoBorder" position="%d,0" size="440,%d" name="globalsConfigScreen" >
			<widget render="Label" source="title" position="20,80" size="400,26" valign="center" zPosition="5" transparent="0" foregroundColor="#fcc000" font="Regular;22"/>
			<widget name="config" position="20,130" size="400,200" scrollbarMode="showOnDemand" />
			<eLabel text="" position="20,450" size="400,2" transparent="0" backgroundColor="#ffffff" />
			<ePixmap name="ButtonRed" pixmap="skin_default/buttons/red.png" position="20,460" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonRedtext" position="20,460" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="160,460" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonGreentext" position="160,460" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
		</screen>""" % (dlg_xh(440))

	def __init__(self, session):
		self.skin = globalsConfigScreen.skin
		self.session = session
		Screen.__init__(self, session)

		list = []
		list.append(getConfigListEntry(_("Show Plugin in Extensions Menu"), config.plugins.NcamStatus.extMenu))
		list.append(getConfigListEntry(_("X-Offset (move left)"), config.plugins.NcamStatus.xOffset))
		list.append(getConfigListEntry(_("ECM Time in \"connected\" Dialog"), config.plugins.NcamStatus.useECM))
		list.append(getConfigListEntry(_("Server address always in IP Format"), config.plugins.NcamStatus.useIP))
		list.append(getConfigListEntry(_("Use Picons"), config.plugins.NcamStatus.usePicons))
		list.append(getConfigListEntry(_("Picons Path"), config.plugins.NcamStatus.PiconPath))
		ConfigListScreen.__init__(self, list, session = session)

		self["title"] = StaticText(_("Ncam Status globals Setup"))
		self["ButtonRedtext"] = StaticText(_("return"))
		self["ButtonGreentext"] = StaticText(_("save"))
		self["actions"] = ActionMap(["OkCancelActions", "ColorActions"],
		{
			"red": self.Exit,
			"green": self.Save,
			"cancel": self.Exit
		}, -1)
		self.onLayoutFinish.append(self.LayoutFinished)

	def LayoutFinished(self):
		x,h = dlg_xh(self.instance.size().width())
		self.instance.move(ePoint(x, 0))

	def Save(self):
		for x in self["config"].list:
			x[1].save()
		self.close()

	def Exit(self):
		for x in self["config"].list:
			x[1].cancel()
		self.close()

class ncamServer:
	serverName = "localhost"
	serverIP   = "127.0.0.1"
	serverPort = "8181"
	username   = "username"
	password   = "password"
	useSSL     = False

CFG = resolveFilename(SCOPE_CURRENT_PLUGIN, "Extensions/NcamStatus/NcamStatus.cfg")

def readCFG():
	cfg = None
	ncamServers = []
	try:
		cfg = file(CFG, "r")
	except:
		pass
	if cfg:
		print "[NcamStatus] reading config file..."
		d = cfg.read()
		cfg.close()
		for line in d.splitlines():
			v = line.strip().split(' ')
			if len(v) == 6:
				tmp = ncamServer()
				tmp.username = v[0]
				tmp.password = v[1]
				tmp.serverIP = v[2]
				tmp.serverPort = v[3]
				tmp.serverName = v[4]
				tmp.useSSL = bool(int(v[5]))
				ncamServers.append(tmp)
	if len(ncamServers) == 0:
		print "[NcamStatus] no config file found, using defaults..."
		tmp = ncamServer()
		ncamServers.append(tmp)
	return ncamServers

def writeCFG(ncamServers):
	cfg = file(CFG, "w")
	print "[NcamStatus] writing datfile..."
	for line in ncamServers:
		cfg.write(line.username+' ')
		cfg.write(line.password+' ')
		cfg.write(line.serverIP+' ')
		cfg.write(line.serverPort+' ')
		cfg.write(line.serverName+' ')
		cfg.write(str(int(line.useSSL))+'\n')
	cfg.close()

class NcamServerEntryList(MenuList):
	def __init__(self, list, enableWrapAround = True):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setFont(1, gFont("Regular", 18))
		self.pic0 = LoadPixmap(cached=True, path=resolveFilename(SCOPE_SKIN, "skin_default/icons/lock_off.png"))
		self.pic1 = LoadPixmap(cached=True, path=resolveFilename(SCOPE_SKIN, "skin_default/icons/lock_on.png"))

	def postWidgetCreate(self, instance):
		MenuList.postWidgetCreate(self, instance)
		instance.setItemHeight(30)

	def makeList(self, index):
		self.list = []
		ncamServers = readCFG()
		for cnt, i in enumerate(ncamServers):
			res = [i]
			if cnt == index:
				if self.pic1:
					res.append((eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 5, 1, 25, 24, self.pic1))
				else:
					res.append((eListboxPythonMultiContent.TYPE_TEXT, 5, 3, 25, 24, 1, RT_HALIGN_LEFT|RT_VALIGN_CENTER, 'x'))
			else:
				if self.pic0:
					res.append((eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 5, 1, 25, 24, self.pic0))
				else:
					res.append((eListboxPythonMultiContent.TYPE_TEXT, 5, 3, 25, 24, 1, RT_HALIGN_LEFT|RT_VALIGN_CENTER, ' '))
			res.append((eListboxPythonMultiContent.TYPE_TEXT,  40, 3, 120, 24, 1, RT_HALIGN_LEFT|RT_VALIGN_CENTER, i.serverName))
			res.append((eListboxPythonMultiContent.TYPE_TEXT, 165, 3, 200, 24, 1, RT_HALIGN_LEFT|RT_VALIGN_CENTER, i.serverIP))
			#res.append((eListboxPythonMultiContent.TYPE_TEXT, 410, 3,  65, 24, 1, RT_HALIGN_LEFT|RT_VALIGN_CENTER, i.serverPort))
			if i.useSSL: tx = "SSL"
			else: tx = ""
			res.append((eListboxPythonMultiContent.TYPE_TEXT, 370, 3, 30, 24, 1, RT_HALIGN_LEFT|RT_VALIGN_CENTER, tx))
			self.list.append(res)
		self.l.setList(self.list)
		self.moveToIndex(index)

# NcamServerEntriesListConfigScreen...
class NcamServerEntriesListConfigScreen(Screen):
	skin = """
		<screen flags="wfNoBorder" position="%d,0" size="440,%d" name="NcamServerEntriesListConfigScreen" >
			<widget render="Label" source="title" position="20,80" size="360,26" valign="center" zPosition="5" transparent="0" foregroundColor="#fcc000" font="Regular;22"/>
			<widget name="list" position="20,130" size="400,288" scrollbarMode="showOnDemand" />
			<eLabel text="" position="20,450" size="400,2" transparent="0" backgroundColor="#ffffff" />
			<ePixmap name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="10,460" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonGreentext" position="10,460" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonYellow" pixmap="skin_default/buttons/yellow.png" position="150,460" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonYellowtext" position="150,460" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonBlue" pixmap="skin_default/buttons/blue.png" position="290,460" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonBluetext" position="290,460" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
		</screen>""" % (dlg_xh(440))

	def __init__(self, session):
		self.skin = NcamServerEntriesListConfigScreen.skin
		self.session = session
		Screen.__init__(self, session)

		self["list"] = NcamServerEntryList([])
		self["list"].makeList(config.plugins.NcamStatus.lastServer.value)

		self["title"] = StaticText(_("Ncam Servers"))
		self["ButtonGreentext"] = StaticText(_("new"))
		self["ButtonYellowtext"] = StaticText(_("edit"))
		self["ButtonBluetext"] = StaticText(_("delete"))

		self["actions"] = ActionMap(["OkCancelActions", "ColorActions"],
		{
			"green": self.keyNew,
			"yellow": self.keyEdit,
			"blue": self.keyDelete,
			"ok": self.keyOk,
			"cancel": self.keyClose
		}, -1)

		self.onLayoutFinish.append(self.LayoutFinished)

	def LayoutFinished(self):
		x,h = dlg_xh(self.instance.size().width())
		self.instance.move(ePoint(x, 0))

	def updateEntrys(self):
		self["list"].makeList(config.plugins.NcamStatus.lastServer.value)

	def keyNew(self):
		self.session.openWithCallback(self.updateEntrys, NcamServerEntryConfigScreen, None, -1)

	def keyEdit(self):
		try:
			entry = self["list"].l.getCurrentSelection()[0]
		except:
			entry = None
		if entry:
			self.session.openWithCallback(self.updateEntrys, NcamServerEntryConfigScreen, entry, self["list"].getSelectionIndex())

	def keyDelete(self):
		try:
			self.index = self["list"].getSelectionIndex()
		except:
			self.index = -1
		if self.index > -1:
			if self.index == config.plugins.NcamStatus.lastServer.value:
				print "[NcamStatus] you can not delete the active entry..."
				return
		message = _("Do you really want to delete this entry?")
		msg = self.session.openWithCallback(self.Confirmed, MessageBox, message)
		msg.setTitle("Ncam Status")


	def Confirmed(self, confirmed):
		if not confirmed:
			return
		ncamServers = readCFG()
		del ncamServers[self.index]
		writeCFG(ncamServers)
		if self.index < config.plugins.NcamStatus.lastServer.value:
			config.plugins.NcamStatus.lastServer.value -= 1
		self.updateEntrys()

	def keyOk(self):
		try:
			entry = self["list"].l.getCurrentSelection()[0]
		except:
			entry = None
		if entry:
			config.plugins.NcamStatus.lastServer.value = self["list"].getSelectionIndex()
			config.plugins.NcamStatus.lastServer.save()
			self.close(entry)

	def keyClose(self):
		self.close(False)


# NcamServerEntryConfigScreen...
class NcamServerEntryConfigScreen(Screen, ConfigListScreen):
	skin = """
		<screen flags="wfNoBorder" position="%d,0" size="440,%d" name="NcamServerEntryConfigScreen" >
			<widget render="Label" source="title" position="20,80" size="400,26" valign="center" zPosition="5" transparent="0" foregroundColor="#fcc000" font="Regular;22"/>
			<widget name="config" position="20,130" size="400,200" scrollbarMode="showOnDemand" />
			<eLabel text="" position="20,450" size="400,2" transparent="0" backgroundColor="#ffffff" />
			<ePixmap name="ButtonRed" pixmap="skin_default/buttons/red.png" position="20,460" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonRedtext" position="20,460" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="160,460" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonGreentext" position="160,460" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
		</screen>""" % (dlg_xh(440))

	def __init__(self, session, entry, index):
		self.skin = NcamServerEntryConfigScreen.skin
		self.session = session
		Screen.__init__(self, session)

		if entry == None:
			entry = ncamServer()
		self.index = index

		# Server Adresse IP-Format oder TextFormat?
		serverIP = self.isIPaddress(entry.serverIP)
		if serverIP and config.plugins.NcamStatus.useIP.value:
			self.isIP = True
		else:
			self.isIP = False

		serverPort = int(entry.serverPort)

		self.serverNameConfigEntry = NoSave(ConfigText(default = entry.serverName, fixed_size = False, visible_width=20))
		if self.isIP:
			self.serverIPConfigEntry = NoSave(ConfigIP( default = serverIP, auto_jump=True))
		else:
			self.serverIPConfigEntry = NoSave(ConfigText(default = entry.serverIP, fixed_size = False, visible_width=20))
			self.serverIPConfigEntry.setUseableChars(u'1234567890aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ.-_')
		self.portConfigEntry       = NoSave(ConfigInteger(default = serverPort, limits=(0,65536)))
		self.usernameConfigEntry   = NoSave(ConfigText(default = entry.username, fixed_size = False, visible_width=20))
		self.passwordConfigEntry   = NoSave(ConfigPassword(default = entry.password, fixed_size = False))
		self.useSSLConfigEntry     = NoSave(ConfigYesNo(entry.useSSL))

		ConfigListScreen.__init__(self, [], session = session)
		self.createSetup()

		self["title"] = StaticText(_("Ncam Server Setup"))
		self["ButtonRedtext"] = StaticText(_("return"))
		self["ButtonGreentext"] = StaticText(_("save"))
		self["actions"] = ActionMap(["OkCancelActions", "ColorActions"],
		{
			"red": self.Close,
			"green": self.Save,
			"cancel": self.Close
		}, -1)

		self.onLayoutFinish.append(self.LayoutFinished)

	def LayoutFinished(self):
		x,h = dlg_xh(self.instance.size().width())
		self.instance.move(ePoint(x, 0))

	def createSetup(self):
		self.list = []
		self.list.append(getConfigListEntry(_("Ncam Server Name"), self.serverNameConfigEntry))
		self.list.append(getConfigListEntry(_("Ncam Server Address"), self.serverIPConfigEntry))
		self.list.append(getConfigListEntry(_("Port"), self.portConfigEntry))
		self.list.append(getConfigListEntry(_("Username (httpuser)"), self.usernameConfigEntry))
		self.list.append(getConfigListEntry(_("Password (httppwd)"), self.passwordConfigEntry))
		self.list.append(getConfigListEntry(_("use SSL"), self.useSSLConfigEntry))
		self["config"].setList(self.list)

	def isIPaddress(self, txt):
		theIP = txt.split('.')
		if len(theIP) != 4:
			return False
		serverIP = []
		for x in theIP:
			try:
				serverIP.append(int(x))
			except:
				return False
		return serverIP

	def Close(self):
		self.close()

	def Save(self):
		entry = ncamServer()
		entry.username   = self.usernameConfigEntry.value
		entry.password   = self.passwordConfigEntry.value
		entry.serverName = self.serverNameConfigEntry.value
		if self.isIP:
			entry.serverIP   = "%d.%d.%d.%d" % tuple(self.serverIPConfigEntry.value)
		else:
			entry.serverIP   = self.serverIPConfigEntry.value
		entry.serverPort = str(self.portConfigEntry.value)
		entry.useSSL     = self.useSSLConfigEntry.value
		ncamServers = readCFG()
		if self.index == -1:
			ncamServers.append(entry)
		else:
			ncamServers[self.index] = entry
		writeCFG(ncamServers)
		self.close()

from os import path, listdir
class piconLoader:
	def __init__(self, Path):
		self.hasLoaded = False
		if path.exists(Path):
			self.Path = Path
			self.picons = {}
			for fname in listdir(self.Path):
				items = fname.split('_')
				# Nur Datei mit sref Namen, also picon png...
				if len(items) > 8:
					#t  = self.picons.get(items[3], 'n/a')
					#if t != 'n/a':
					#	print fname, t
					self.picons[items[3]] = fname
			if len(self.picons) == 0:
				print "[NcamStatus] ERROR: no picons found!"
				return
			else:
				print "[NcamStatus] %d picons found..." % len(self.picons)
				self.hasLoaded = True
				return
		else:
			print "[NcamStatus] ERROR: wrong picons path!"
			return

	def getPicon(self, srvid):
		# FTA Sender oder nicht gefunden...
		if srvid == '0000':
			return None

		# keine fuehrenden Nullen...
		while srvid[0] == '0':
			srvid = srvid[1:]

		# Dateinamen aus dict holen...
		fname = self.picons.get(srvid, 'n/a')
		if fname != 'n/a':
			piconpath = self.Path+'/'+fname
			return LoadPixmap(cached=True, path=piconpath)
		else:
			return LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_PLUGIN, "Extensions/NcamStatus/icons/unknown.png"))

